@@ -1,10 +1,8 @@
-/* $Cambridge: exim/src/src/filter.c,v 1.15 2009/11/16 19:50:37 nm4 Exp $ */
-
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2004 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -21,9 +19,10 @@ need to declare the structures first because some of them are recursive. */
 
 struct filter_cmd;
 struct condition_block;
+struct alias;
 
 union argtypes {
-  struct string_item     *a;
+  struct alias           *a;
   BOOL                    b;
   struct condition_block *c;
   struct filter_cmd      *f;
@@ -49,6 +48,11 @@ typedef struct condition_block {
   union argtypes right;
 } condition_block;
 
+typedef struct alias {
+  struct alias *next;
+  uschar *name;
+} alias;
+
 /* Miscellaneous other declarations */
 
 static uschar **error_pointer;
@@ -518,7 +522,7 @@ for (;;)
       c->type = cond_personal;
       for (;;)
         {
-        string_item *aa;
+        alias *aa;
         uschar *saveptr = ptr;
         ptr = nextword(ptr, buffer, sizeof(buffer), TRUE);
         if (*error_pointer != NULL) break;
@@ -529,8 +533,8 @@ for (;;)
           }
         ptr = nextitem(ptr, buffer, sizeof(buffer), TRUE);
         if (*error_pointer != NULL) break;
-        aa = store_get(sizeof(string_item));
-        aa->text = string_copy(buffer);
+        aa = store_get(sizeof(alias));
+        aa->name = string_copy(buffer);
         aa->next = c->left.a;
         c->left.a = aa;
         }
@@ -1044,13 +1048,6 @@ switch (command)
   case elif_command:
   case else_command:
   case endif_command:
-  if (seen_force || noerror_force)
-    {
-    *error_pointer = string_sprintf("\"seen\", \"unseen\", or \"noerror\" "
-      "near line %d is not followed by a command", line_number);
-    yield = FALSE;
-    }
-
   if (expect_endif > 0)
     had_else_endif = (command == elif_command)? had_elif :
                      (command == else_command)? had_else : had_endif;
@@ -1323,12 +1320,6 @@ switch (command)
 
   case seen_command:
   case unseen_command:
-  if (*ptr == 0)
-    {
-    *error_pointer = string_sprintf("\"seen\" or \"unseen\" "
-      "near line %d is not followed by a command", line_number);
-    yield = FALSE;
-    }
   if (seen_force)
     {
     *error_pointer = string_sprintf("\"seen\" or \"unseen\" repeated "
@@ -1344,12 +1335,6 @@ switch (command)
   /* So does noerror */
 
   case noerror_command:
-  if (*ptr == 0)
-    {
-    *error_pointer = string_sprintf("\"noerror\" "
-      "near line %d is not followed by a command", line_number);
-    yield = FALSE;
-    }
   noerror_force = TRUE;
   was_noerror = TRUE;
   break;
@@ -1417,6 +1402,60 @@ return TRUE;
 
 
 
+/*************************************************
+*   Test a header for the "personal" condition   *
+*************************************************/
+
+/* A bit more efficient than doing a lot of explicit expansions. We pass
+in a chain of alias blocks and a variable-length list of explicit text
+strings.
+
+Arguments:
+  s          header name, including the trailing colon
+  cond       value to return if the header contains any of the strings
+               or aliases
+  aliases    points to a chain of alias blocks
+  count      number of strings
+  ...        the strings
+
+Returns:     cond if the header exists and contains one of the aliases or
+               strings; otherwise !cond
+*/
+
+static BOOL
+test_header(uschar *s, BOOL cond, alias *aliases, int count, ...)
+{
+header_line *h;
+int slen = Ustrlen(s);
+
+for (h = header_list; h != NULL; h = h->next)
+  {
+  int i;
+  alias *a;
+  va_list ap;
+
+  if (h->type == htype_old || slen > h->slen ||
+    strncmpic(s, h->text, slen) != 0)
+      continue;
+
+  for (a = aliases; a != NULL; a = a->next)
+    if (strstric(h->text + slen, a->name, FALSE) != NULL) return cond;
+
+  va_start(ap, count);
+  for (i = 0; i < count; i++)
+    {
+    uschar *p = va_arg(ap, uschar *);
+    if (strstric(h->text + slen, p, FALSE) != NULL) return cond;
+    }
+  va_end(ap);
+  }
+
+return !cond;
+}
+
+
+
+
 /*************************************************
 *             Test a condition                   *
 *************************************************/
@@ -1433,7 +1472,7 @@ Returns:         TRUE if the condition is met
 static BOOL
 test_condition(condition_block *c, BOOL toplevel)
 {
-BOOL yield = FALSE;
+BOOL yield;
 const pcre *re;
 uschar *exp[2], *p, *pp;
 const uschar *regcomp_error = NULL;
@@ -1457,12 +1496,59 @@ switch (c->type)
           test_condition(c->right.c, FALSE));
   break;
 
-  /* The personal test is meaningless in a system filter. The tests are now in
-  a separate function (so Sieve can use them). However, an Exim filter does not
-  scan Cc: (hence the FALSE argument). */
+  /* The personal test is meaningless in a system filter */
 
   case cond_personal:
-  yield = system_filtering? FALSE : filter_personal(c->left.a, FALSE);
+  if (system_filtering) yield = FALSE; else
+    {
+    alias *aliases = c->left.a;
+    uschar *self, *self_from, *self_to;
+    uschar *psself = NULL;
+    uschar *psself_from = NULL, *psself_to = NULL;
+    int to_count = 2;
+    int from_count = 5;
+
+    self = string_sprintf("%s@%s", deliver_localpart, deliver_domain);
+    self_from = rewrite_one(self, rewrite_from, NULL, FALSE, US"",
+      global_rewrite_rules);
+    self_to   = rewrite_one(self, rewrite_to, NULL, FALSE, US"",
+      global_rewrite_rules);
+    if (self_from == NULL) self_from = self;
+    if (self_to == NULL) self_to = self;
+
+    /* If there's a prefix or suffix set, we must include the prefixed/
+    suffixed version of the local part in the tests. */
+
+    if (deliver_localpart_prefix != NULL || deliver_localpart_suffix != NULL)
+      {
+      psself = string_sprintf("%s%s%s@%s",
+        (deliver_localpart_prefix == NULL)? US"" : deliver_localpart_prefix,
+        deliver_localpart,
+        (deliver_localpart_suffix == NULL)? US"" : deliver_localpart_suffix,
+        deliver_domain);
+      psself_from = rewrite_one(psself, rewrite_from, NULL, FALSE, US"",
+        global_rewrite_rules);
+      psself_to   = rewrite_one(psself, rewrite_to, NULL, FALSE, US"",
+        global_rewrite_rules);
+      if (psself_from == NULL) self_from = psself;
+      if (psself_to == NULL) self_to = psself;
+      to_count += 2;
+      from_count += 2;
+      }
+
+    /* Do all the necessary tests; the counts are adjusted for {pre,suf}fix */
+
+    yield = test_header(US"to:", TRUE, aliases, to_count, self, self_to,
+              psself, psself_to) &&
+            test_header(US"from:", FALSE, aliases, from_count, "server@",
+              "daemon@", "root@", self, self_from, psself, psself_from) &&
+            test_header(US"subject:", FALSE, NULL, 1, "circular") &&
+            test_header(US"auto-submitted:", FALSE, NULL, 1, "auto-") &&
+            test_header(US"precedence:", FALSE, NULL, 1, "bulk") &&
+            test_header(US"precedence:", FALSE, NULL, 1, "list") &&
+            test_header(US"precedence:", FALSE, NULL, 1, "junk") &&
+            (sender_address == NULL || sender_address[0] != 0);
+    }
   break;
 
   case cond_delivered:
@@ -1481,7 +1567,7 @@ switch (c->type)
   and filter testing and verification. */
 
   case cond_firsttime:
-  yield = filter_test != FTEST_NONE || message_id[0] == 0 || deliver_firsttime;
+  yield = filter_test != NULL || message_id[0] == 0 || deliver_firsttime;
   break;
 
   /* Only TRUE if a message is actually being processed; FALSE for address
@@ -1522,7 +1608,7 @@ switch (c->type)
 
     if (filter_thisaddress != NULL)
       {
-      if ((filter_test != FTEST_NONE && debug_selector != 0) ||
+      if ((filter_test != NULL && debug_selector != 0) ||
           (debug_selector & D_filter) != 0)
         {
         indent();
@@ -1597,7 +1683,7 @@ switch (c->type)
 
     case cond_matches:
     case cond_MATCHES:
-    if ((filter_test != FTEST_NONE && debug_selector != 0) ||
+    if ((filter_test != NULL && debug_selector != 0) ||
         (debug_selector & D_filter) != 0)
       {
       debug_printf("Match expanded arguments:\n");
@@ -1640,7 +1726,7 @@ switch (c->type)
   break;
   }
 
-if ((filter_test != FTEST_NONE && debug_selector != 0) ||
+if ((filter_test != NULL && debug_selector != 0) ||
     (debug_selector & D_filter) != 0)
   {
   indent();
@@ -1748,7 +1834,7 @@ while (commands != NULL)
       }
 
     filter_n[n[1]] += n[0];
-    if (filter_test != FTEST_NONE) printf("Add %d to n%d\n", n[0], n[1]);
+    if (filter_test != NULL) printf("Add %d to n%d\n", n[0], n[1]);
     break;
 
     /* A deliver command's argument must be a valid address. Its optional
@@ -1796,7 +1882,7 @@ while (commands != NULL)
 
     /* Test case: report what would happen */
 
-    if (filter_test != FTEST_NONE)
+    if (filter_test != NULL)
       {
       indent();
       printf("%seliver message to: %s%s%s%s\n",
@@ -1837,7 +1923,7 @@ while (commands != NULL)
 
     /* Test case: report what would happen */
 
-    if (filter_test != FTEST_NONE)
+    if (filter_test != NULL)
       {
       indent();
       if (mode < 0)
@@ -1853,12 +1939,11 @@ while (commands != NULL)
 
     else
       {
-      if (s[0] != '/' && (filter_options & RDO_PREPEND_HOME) != 0 &&
-          deliver_home != NULL && deliver_home[0] != 0)
-        s = string_sprintf("%s/%s", deliver_home, s);
       DEBUG(D_filter) debug_printf("Filter: %ssave message to: %s%s\n",
         (commands->seen)? "" : "unseen ", s,
         commands->noerror? " (noerror)" : "");
+      if (s[0] != '/' && deliver_home != NULL && deliver_home[0] != 0)
+        s = string_sprintf("%s/%s", deliver_home, s);
 
       /* Create the new address and add it to the chain, setting the
       af_pfr and af_file flags, the af_ignore_error flag if necessary, and the
@@ -1875,7 +1960,7 @@ while (commands != NULL)
 
     case pipe_command:
     s = string_copy(commands->args[0].u);
-    if (filter_test != FTEST_NONE)
+    if (filter_test != NULL)
       {
       indent();
       printf("%sipe message to: %s%s\n", (commands->seen)?
@@ -1927,11 +2012,11 @@ while (commands != NULL)
     if (log_mode == -1) log_mode = 0600;
     if (log_fd >= 0)
       {
-      (void)close(log_fd);
+      close(log_fd);
       log_fd = -1;
       }
     log_filename = expargs[0];
-    if (filter_test != FTEST_NONE)
+    if (filter_test != NULL)
       {
       indent();
       printf("%sogfile %s\n", (commands->seen)? "Seen l" : "L", log_filename);
@@ -1941,7 +2026,7 @@ while (commands != NULL)
     case logwrite_command:
     s = expargs[0];
 
-    if (filter_test != FTEST_NONE)
+    if (filter_test != NULL)
       {
       indent();
       printf("%sogwrite \"%s\"\n", (commands->seen)? "Seen l" : "L",
@@ -2003,7 +2088,7 @@ while (commands != NULL)
       int subtype = commands->args[1].i;
       s = expargs[0];
 
-      if (filter_test != FTEST_NONE)
+      if (filter_test != NULL)
         printf("Headers %s \"%s\"\n", (subtype == TRUE)? "add" :
           (subtype == FALSE)? "remove" : "charset", string_printing(s));
 
@@ -2015,19 +2100,31 @@ while (commands != NULL)
           header_add(htype_other, "%s%s", s, (s[Ustrlen(s)-1] == '\n')?
             "" : "\n");
           header_last->type = header_checkname(header_last, FALSE);
-          if (header_last->type >= 'a') header_last->type = htype_other;
           }
         }
 
       else if (subtype == FALSE)
         {
-        int sep = 0;
-        uschar *ss;
-        uschar *list = s;
-        uschar buffer[128];
-        while ((ss = string_nextinlist(&list, &sep, buffer, sizeof(buffer)))
-               != NULL)
-          header_remove(0, ss);
+        header_line *h;
+        for (h = header_list; h != NULL; h = h->next)
+          {
+          int sep = 0;
+          uschar *ss, *tt;
+          uschar *list = s;
+          uschar buffer[128];
+          if (h->type == htype_old) continue;
+          while ((ss = string_nextinlist(&list, &sep, buffer, sizeof(buffer)))
+                 != NULL)
+            {
+            int len = Ustrlen(ss);
+            if (strncmpic(h->text, ss, len) != 0) continue;
+            tt = h->text + len;
+            while (*tt == ' ' || *tt == '\t') tt++;
+            if (*tt == ':') break;
+            }
+          if (ss == NULL) continue;
+          h->type = htype_old;
+          }
         }
 
       /* This setting lasts only while the filter is running; on exit, the
@@ -2062,7 +2159,7 @@ while (commands != NULL)
     fmsg = string_printing(fmsg);
     *error_pointer = fmsg;
 
-    if (filter_test != FTEST_NONE)
+    if (filter_test != NULL)
       {
       indent();
       printf("%c%s text \"%s\"\n", toupper(ff_name[0]), ff_name+1, fmsg);
@@ -2071,7 +2168,7 @@ while (commands != NULL)
     return ff_ret;
 
     case finish_command:
-    if (filter_test != FTEST_NONE)
+    if (filter_test != NULL)
       {
       indent();
       printf("%sinish\n", (commands->seen)? "Seen f" : "F");
@@ -2111,7 +2208,7 @@ while (commands != NULL)
     case vacation_command:
     if (return_path == NULL || return_path[0] == 0)
       {
-      if (filter_test != FTEST_NONE)
+      if (filter_test != NULL)
         printf("%s command ignored because return_path is empty\n",
           command_list[commands->command]);
       else DEBUG(D_filter) debug_printf("%s command ignored because return_path "
@@ -2122,8 +2219,8 @@ while (commands != NULL)
     /* Check the contents of the strings. The type of string can be deduced
     from the value of i.
 
-    . If i is equal to mailarg_index_text it's a text string for the body,
-      where anything goes.
+    . If i is equal to mailarg_index_text its a text string for the body, where
+      anything goes.
 
     . If i is > mailarg_index_text, we are dealing with a file name, which
       cannot contain non-printing characters.
@@ -2186,6 +2283,7 @@ while (commands != NULL)
                   string_printing(s), command_list[commands->command]);
                 return FF_ERROR;
                 }
+              pp++;
               }
             p = pp;
             }
@@ -2199,7 +2297,7 @@ while (commands != NULL)
 
     /* Proceed with mail or vacation command */
 
-    if (filter_test != FTEST_NONE)
+    if (filter_test != NULL)
       {
       uschar *to = commands->args[mailarg_index_to].u;
       indent();
@@ -2226,17 +2324,12 @@ while (commands != NULL)
     else
       {
       uschar *tt;
-      uschar *log_addr = NULL;
       uschar *to = commands->args[mailarg_index_to].u;
-      int size = 0;
-      int ptr = 0;
-      int badflag = 0;
-
       if (to == NULL) to = expand_string(US"$reply_address");
       while (isspace(*to)) to++;
 
-      for (tt = to; *tt != 0; tt++)     /* Get rid of newlines */
-        if (*tt == '\n') *tt = ' ';
+      for (tt = to; *tt != 0; tt++)     /* Get rid of newlines so that */
+        if (*tt == '\n') *tt = ' ';     /* the eventual log line is OK */
 
       DEBUG(D_filter)
         {
@@ -2259,59 +2352,10 @@ while (commands != NULL)
           }
         }
 
-      /* Create the "address" for the autoreply. This is used only for logging,
-      as the actual recipients are extracted from the To: line by -t. We use the
-      same logic here to extract the working addresses (there may be more than
-      one). Just in case there are a vast number of addresses, stop when the
-      string gets too long. */
+      /* Create the "address" for the autoreply */
 
-      tt = to;
-      while (*tt != 0)
-        {
-        uschar *ss = parse_find_address_end(tt, FALSE);
-        uschar *recipient, *errmess;
-        int start, end, domain;
-        int temp = *ss;
-
-        *ss = 0;
-        recipient = parse_extract_address(tt, &errmess, &start, &end, &domain,
-          FALSE);
-        *ss = temp;
-
-        /* Ignore empty addresses and errors; an error will occur later if
-        there's something really bad. */
-
-        if (recipient != NULL)
-          {
-          log_addr = string_cat(log_addr, &size, &ptr,
-            (log_addr == NULL)? US">" : US",", 1);
-          log_addr = string_cat(log_addr, &size, &ptr, recipient,
-            Ustrlen(recipient));
-          }
-
-        /* Check size */
-
-        if (ptr > 256)
-          {
-          log_addr = string_cat(log_addr, &size, &ptr, US", ...", 5);
-          break;
-          }
-
-        /* Move on past this address */
-
-        tt = ss + (*ss? 1:0);
-        while (isspace(*tt)) tt++;
-        }
-
-      if (log_addr == NULL)
-        {
-        log_addr = string_sprintf(">**bad-reply**");
-        badflag = af_bad_reply;
-        }
-      else log_addr[ptr] = 0;
-
-      addr = deliver_make_addr(log_addr, FALSE);
-      setflag(addr, (af_pfr|badflag));
+      addr = deliver_make_addr(string_sprintf(">%.256s", to), FALSE);
+      setflag(addr, af_pfr);
       if (commands->noerror) setflag(addr, af_ignore_error);
       addr->next = *generated;
       *generated = addr;
@@ -2323,7 +2367,6 @@ while (commands != NULL)
       addr->reply->expand_forbid = expand_forbid;
       addr->reply->return_message =
         commands->args[mailarg_index_return].u != NULL;
-      addr->reply->once_repeat = 0;
 
       if (commands->args[mailarg_index_once_repeat].u != NULL)
         {
@@ -2351,10 +2394,10 @@ while (commands != NULL)
     break;
 
     case testprint_command:
-    if (filter_test != FTEST_NONE || (debug_selector & D_filter) != 0)
+    if (filter_test != NULL || (debug_selector & D_filter) != 0)
       {
       uschar *s = string_printing(expargs[0]);
-      if (filter_test == FTEST_NONE)
+      if (filter_test == NULL)
         debug_printf("Filter: testprint: %s\n", s);
       else
         printf("Testprint: %s\n", s);
@@ -2369,132 +2412,6 @@ return filter_delivered? FF_DELIVERED : FF_NOTDELIVERED;
 
 
 
-/*************************************************
-*        Test for a personal message             *
-*************************************************/
-
-/* This function is global so that it can also be called from the code that
-implements Sieve filters.
-
-Arguments:
-  aliases    a chain of aliases
-  scan_cc    TRUE if Cc: and Bcc: are to be scanned (Exim filters do not)
-
-Returns:     TRUE if the message is deemed to be personal
-*/
-
-BOOL
-filter_personal(string_item *aliases, BOOL scan_cc)
-{
-uschar *self, *self_from, *self_to;
-uschar *psself = NULL, *psself_from = NULL, *psself_to = NULL;
-void *reset_point = store_get(0);
-BOOL yield;
-header_line *h;
-int to_count = 2;
-int from_count = 9;
-
-/* If any header line in the message is a defined "List-" header field, it is
-not a personal message. We used to check for any header line that started with
-"List-", but this was tightened up for release 4.54. The check is now for
-"List-Id", defined in RFC 2929, or "List-Help", "List-Subscribe", "List-
-Unsubscribe", "List-Post", "List-Owner" or "List-Archive", all of which are
-defined in RFC 2369. We also scan for "Auto-Submitted"; if it is found to
-contain any value other than "no", the message is not personal (RFC 3834).
-Previously the test was for "auto-". */
-
-for (h = header_list; h != NULL; h = h->next)
-  {
-  uschar *s;
-  if (h->type == htype_old) continue;
-
-  if (strncmpic(h->text, US"List-", 5) == 0)
-    {
-    s = h->text + 5;
-    if (strncmpic(s, US"Id:", 3) == 0 ||
-        strncmpic(s, US"Help:", 5) == 0 ||
-        strncmpic(s, US"Subscribe:", 10) == 0 ||
-        strncmpic(s, US"Unsubscribe:", 12) == 0 ||
-        strncmpic(s, US"Post:", 5) == 0 ||
-        strncmpic(s, US"Owner:", 6) == 0 ||
-        strncmpic(s, US"Archive:", 8) == 0)
-      return FALSE;
-    }
-
-  else if (strncmpic(h->text, US"Auto-submitted:", 15) == 0)
-    {
-    s = h->text + 15;
-    while (isspace(*s)) s++;
-    if (strncmpic(s, US"no", 2) != 0) return FALSE;
-    s += 2;
-    while (isspace(*s)) s++;
-    if (*s != 0) return FALSE;
-    }
-  }
-
-/* Set up "my" address */
-
-self = string_sprintf("%s@%s", deliver_localpart, deliver_domain);
-self_from = rewrite_one(self, rewrite_from, NULL, FALSE, US"",
-  global_rewrite_rules);
-self_to   = rewrite_one(self, rewrite_to, NULL, FALSE, US"",
-  global_rewrite_rules);
-
-
-if (self_from == NULL) self_from = self;
-if (self_to == NULL) self_to = self;
-
-/* If there's a prefix or suffix set, we must include the prefixed/
-suffixed version of the local part in the tests. */
-
-if (deliver_localpart_prefix != NULL || deliver_localpart_suffix != NULL)
-  {
-  psself = string_sprintf("%s%s%s@%s",
-    (deliver_localpart_prefix == NULL)? US"" : deliver_localpart_prefix,
-    deliver_localpart,
-    (deliver_localpart_suffix == NULL)? US"" : deliver_localpart_suffix,
-    deliver_domain);
-  psself_from = rewrite_one(psself, rewrite_from, NULL, FALSE, US"",
-    global_rewrite_rules);
-  psself_to   = rewrite_one(psself, rewrite_to, NULL, FALSE, US"",
-    global_rewrite_rules);
-  if (psself_from == NULL) psself_from = psself;
-  if (psself_to == NULL) psself_to = psself;
-  to_count += 2;
-  from_count += 2;
-  }
-
-/* Do all the necessary tests; the counts are adjusted for {pre,suf}fix */
-
-yield =
-  (
-  header_match(US"to:", TRUE, TRUE, aliases, to_count, self, self_to, psself,
-               psself_to) ||
-    (scan_cc &&
-       (
-       header_match(US"cc:", TRUE, TRUE, aliases, to_count, self, self_to,
-                             psself, psself_to)
-       ||
-       header_match(US"bcc:", TRUE, TRUE, aliases, to_count, self, self_to,
-                              psself, psself_to)
-       )
-    )
-  ) &&
-
-  header_match(US"from:", TRUE, FALSE, aliases, from_count, "^server@",
-    "^daemon@", "^root@", "^listserv@", "^majordomo@", "^.*?-request@",
-    "^owner-[^@]+@", self, self_from, psself, psself_from) &&
-
-  header_match(US"precedence:", FALSE, FALSE, NULL, 3, "bulk","list","junk") &&
-
-  (sender_address == NULL || sender_address[0] != 0);
-
-store_reset(reset_point);
-return yield;
-}
-
-
-
 /*************************************************
 *            Interpret a mail filter file        *
 *************************************************/
@@ -2560,7 +2477,7 @@ ptr = nextsigchar(ptr, TRUE);
 if (read_command_list(&ptr, &lastcmdptr, FALSE))
   yield = interpret_commands(commands, generated);
 
-if (filter_test != FTEST_NONE || (debug_selector & D_filter) != 0)
+if (filter_test != NULL || (debug_selector & D_filter) != 0)
   {
   uschar *s = US"";
   switch(yield)
@@ -2593,14 +2510,14 @@ if (filter_test != FTEST_NONE || (debug_selector & D_filter) != 0)
     break;
     }
 
-  if (filter_test != FTEST_NONE) printf("%s\n", CS s);
+  if (filter_test != NULL) printf("%s\n", CS s);
     else debug_printf("%s\n", s);
   }
 
 /* Close the log file if it was opened, and kill off any numerical variables
 before returning. Reset the header decoding charset. */
 
-if (log_fd >= 0) (void)close(log_fd);
+if (log_fd >= 0) close(log_fd);
 expand_nmax = -1;
 filter_running = FALSE;
 headers_charset = save_headers_charset;
