@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2018 */
+/* Copyright (c) University of Cambridge 1995 - 2004 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for matching strings */
@@ -15,8 +15,8 @@
 strings, domains, and local parts. */
 
 typedef struct check_string_block {
-  const uschar *origsubject;           /* caseful; keep these two first, in */
-  const uschar *subject;               /* step with the block below */
+  uschar *origsubject;               /* caseful; keep these two first, in */
+  uschar *subject;                   /* step with the block below */
   int    expand_setup;
   BOOL   use_partial;
   BOOL   caseless;
@@ -28,7 +28,7 @@ typedef struct check_string_block {
 addresses. */
 
 typedef struct check_address_block {
-  const uschar *origaddress;         /* caseful; keep these two first, in */
+  uschar *origaddress;               /* caseful; keep these two first, in */
   uschar *address;                   /* step with the block above */
   int    expand_setup;
   BOOL   caseless;
@@ -72,8 +72,7 @@ Arguments:
                    returns ERROR)
 
 Contents of the argument block:
-  origsubject    the subject in its original casing
-  subject        the subject string to be checked, lowercased if caseless
+  subject        the subject string to be checked
   expand_setup   if < 0, don't set up any numeric expansion variables;
                  if = 0, set $0 to whole subject, and either
                    $1 to what matches * or
@@ -92,13 +91,13 @@ Returns:       OK    if matched
 */
 
 static int
-check_string(void *arg, const uschar *pattern, const uschar **valueptr, uschar **error)
+check_string(void *arg, uschar *pattern, uschar **valueptr, uschar **error)
 {
-const check_string_block *cb = arg;
+check_string_block *cb = (check_string_block *)arg;
 int search_type, partial, affixlen, starflags;
 int expand_setup = cb->expand_setup;
-const uschar *affix;
-uschar *s;
+uschar *affix;
+uschar *s = cb->subject;
 uschar *filename = NULL;
 uschar *keyquery, *result, *semicolon;
 void *handle;
@@ -107,12 +106,6 @@ error = error;  /* Keep clever compilers from complaining */
 
 if (valueptr != NULL) *valueptr = NULL;  /* For non-lookup matches */
 
-/* For regular expressions, use cb->origsubject rather than cb->subject so that
-it works if the pattern uses (?-i) to turn off case-independence, overriding
-"caseless". */
-
-s = string_copy(pattern[0] == '^' ? cb->origsubject : cb->subject);
-
 /* If required to set up $0, initialize the data but don't turn on by setting
 expand_nmax until the match is assured. */
 
@@ -131,7 +124,7 @@ if (pattern[0] == '^')
   {
   const pcre *re = regex_must_compile(pattern, cb->caseless, FALSE);
   return ((expand_setup < 0)?
-           pcre_exec(re, NULL, CCS s, Ustrlen(s), 0, PCRE_EOPT, NULL, 0) >= 0
+           pcre_exec(re, NULL, CS s, Ustrlen(s), 0, PCRE_EOPT, NULL, 0) >= 0
            :
            regex_match_and_setup(re, s, 0, expand_setup)
          )?
@@ -179,9 +172,7 @@ if (cb->at_is_special && pattern[0] == '@')
     int slen = Ustrlen(s);
     if (s[0] != '[' && s[slen-1] != ']') return FAIL;
     for (ip = host_find_interfaces(); ip != NULL; ip = ip->next)
-      if (Ustrncmp(ip->address, s+1, slen - 2) == 0
-            && ip->address[slen - 2] == 0)
-        return OK;
+      if (Ustrncmp(ip->address, s+1, slen - 2) == 0) return OK;
     return FAIL;
     }
 
@@ -192,8 +183,8 @@ if (cb->at_is_special && pattern[0] == '@')
     BOOL prim = FALSE;
     BOOL secy = FALSE;
     BOOL removed = FALSE;
-    const uschar *ss = pattern + 4;
-    const uschar *ignore_target_hosts = NULL;
+    uschar *ss = pattern + 4;
+    uschar *ignore_target_hosts = NULL;
 
     if (strncmpic(ss, US"any", 3) == 0) ss += 3;
     else if (strncmpic(ss, US"primary", 7) == 0)
@@ -219,9 +210,8 @@ if (cb->at_is_special && pattern[0] == '@')
       ignore_target_hosts,
       HOST_FIND_BY_MX,     /* search only for MX, not SRV or A */
       NULL,                /* service name not relevant */
-      NULL,                /* srv_fail_domains not relevant */
-      NULL,                /* mx_fail_domains not relevant */
-      NULL,                /* no dnssec request/require XXX ? */
+      FALSE,               /* no widening */
+      FALSE,               /* no widening */
       NULL,                /* no feedback FQDN */
       &removed);           /* feedback if local removed */
 
@@ -274,20 +264,12 @@ up user@domain for sender rejection). There's a flag to disable it. */
 
 if (!cb->use_partial) partial = -1;
 
-/* Set the parameters for the three different kinds of lookup. */
+/* Set the parameters for the two different kinds of lookup. */
 
 keyquery = semicolon + 1;
 while (isspace(*keyquery)) keyquery++;
 
-if (mac_islookup(search_type, lookup_absfilequery))
-  {
-  filename = keyquery;
-  while (*keyquery != 0 && !isspace(*keyquery)) keyquery++;
-  filename = string_copyn(filename, keyquery - filename);
-  while (isspace(*keyquery)) keyquery++;
-  }
-
-else if (!mac_islookup(search_type, lookup_querystyle))
+if (!mac_islookup(search_type, lookup_querystyle))
   {
   filename = keyquery;
   keyquery = s;
@@ -297,13 +279,14 @@ else if (!mac_islookup(search_type, lookup_querystyle))
 for; partial matching is all handled inside search_find(). Note that there is
 no search_close() because of the caching arrangements. */
 
-if (!(handle = search_open(filename, search_type, 0, NULL, NULL)))
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s", search_error_message);
+handle = search_open(filename, search_type, 0, NULL, NULL);
+if (handle == NULL) log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s",
+  search_error_message);
 result = search_find(handle, filename, keyquery, partial, affix, affixlen,
   starflags, &expand_setup);
 
-if (!result) return f.search_find_defer? DEFER : FAIL;
-if (valueptr) *valueptr = result;
+if (result == NULL) return search_find_defer? DEFER : FAIL;
+if (valueptr != NULL) *valueptr = result;
 
 expand_nmax = expand_setup;
 return OK;
@@ -335,8 +318,8 @@ Returns:       OK    if matched
 */
 
 int
-match_check_string(const uschar *s, const uschar *pattern, int expand_setup,
-  BOOL use_partial, BOOL caseless, BOOL at_is_special, const uschar **valueptr)
+match_check_string(uschar *s, uschar *pattern, int expand_setup,
+  BOOL use_partial, BOOL caseless, BOOL at_is_special, uschar **valueptr)
 {
 check_string_block cb;
 cb.origsubject = s;
@@ -364,7 +347,7 @@ Arguments:
   type         MCL_STRING, MCL_DOMAIN, MCL_HOST, MCL_ADDRESS, or MCL_LOCALPART
 */
 
-static const uschar *
+static uschar *
 get_check_key(void *arg, int type)
 {
 switch(type)
@@ -430,21 +413,19 @@ Returns:       OK    if matched a non-negated item
                FAIL  if expansion force-failed
                FAIL  if matched a negated item
                FAIL  if hit end of list after a non-negated item
-               DEFER if a something deferred or expansion failed
+               DEFER if a lookup deferred or expansion failed
 */
 
 int
-match_check_list(const uschar **listptr, int sep, tree_node **anchorptr,
-  unsigned int **cache_ptr, int (*func)(void *,const uschar *,const uschar **,uschar **),
-  void *arg, int type, const uschar *name, const uschar **valueptr)
+match_check_list(uschar **listptr, int sep, tree_node **anchorptr,
+  unsigned int **cache_ptr, int (*func)(void *,uschar *,uschar **,uschar **),
+  void *arg, int type, uschar *name, uschar **valueptr)
 {
 int yield = OK;
 unsigned int *original_cache_bits = *cache_ptr;
 BOOL include_unknown = FALSE;
 BOOL ignore_unknown = FALSE;
-BOOL include_defer = FALSE;
-BOOL ignore_defer = FALSE;
-const uschar *list;
+uschar *list;
 uschar *sss;
 uschar *ot = NULL;
 uschar buffer[1024];
@@ -457,12 +438,12 @@ HDEBUG(D_any)
   if (listname[0] != 0) ot = string_sprintf("%s in %s?", name, listname);
   }
 
-/* If the list is empty, the answer is no. Skip the debugging output for
-an unnamed list. */
+/* If the list is empty, the answer is no. This case happens only when checking
+named options, so ot will be set. */
 
-if (!*listptr)
+if (*listptr == NULL)
   {
-  HDEBUG(D_lists) if (ot) debug_printf("%s no (option unset)\n", ot);
+  HDEBUG(D_lists) debug_printf("%s no (option unset)\n", ot);
   return FAIL;
   }
 
@@ -478,22 +459,10 @@ if (type >= MCL_NOEXPAND)
   }
 else
   {
-  /* If we are searching a domain list, and $domain is not set, set it to the
-  subject that is being sought for the duration of the expansion. */
-
-  if (type == MCL_DOMAIN && !deliver_domain)
+  list = expand_string(*listptr);
+  if (list == NULL)
     {
-    check_string_block *cb = (check_string_block *)arg;
-    deliver_domain = string_copy(cb->subject);
-    list = expand_cstring(*listptr);
-    deliver_domain = NULL;
-    }
-  else
-    list = expand_cstring(*listptr);
-
-  if (!list)
-    {
-    if (f.expand_string_forcedfail)
+    if (expand_string_forcedfail)
       {
       HDEBUG(D_lists) debug_printf("expansion of \"%s\" forced failure: "
         "assume not in this list\n", *listptr);
@@ -507,14 +476,17 @@ else
 
 /* For an unnamed list, use the expanded version in comments */
 
-HDEBUG(D_any) if (ot == NULL) ot = string_sprintf("%s in \"%s\"?", name, list);
+HDEBUG(D_any)
+  {
+  if (ot == NULL) ot = string_sprintf("%s in \"%s\"?", name, list);
+  }
 
 /* Now scan the list and process each item in turn, until one of them matches,
 or we hit an error. */
 
-while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
+while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
   {
-  uschar * ss = sss;
+  uschar *ss = sss;
 
   /* Address lists may contain +caseful, to restore caseful matching of the
   local part. We have to know the layout of the control block, unfortunately.
@@ -527,8 +499,7 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
       {
       check_address_block *cb = (check_address_block *)arg;
       uschar *at = Ustrrchr(cb->origaddress, '@');
-
-      if (at)
+      if (at != NULL)
         Ustrncpy(cb->address, cb->origaddress, at - cb->origaddress);
       cb->caseless = FALSE;
       continue;
@@ -548,11 +519,10 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
       }
     }
 
-  /* If the host item is "+include_unknown" or "+ignore_unknown", remember it
-  in case there's a subsequent failed reverse lookup. There is similar
-  processing for "defer". */
+  /* If the host item is "+include_unknown", remember it in case there's a
+  subsequent failed reverse lookup. */
 
-  else if (type == MCL_HOST && *ss == '+')
+  else if (type == MCL_HOST)
     {
     if (Ustrcmp(ss, "+include_unknown") == 0)
       {
@@ -566,18 +536,6 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
       include_unknown = FALSE;
       continue;
       }
-    if (Ustrcmp(ss, "+include_defer") == 0)
-      {
-      include_defer = TRUE;
-      ignore_defer = FALSE;
-      continue;
-      }
-    if (Ustrcmp(ss, "+ignore_defer") == 0)
-      {
-      ignore_defer = TRUE;
-      include_defer = FALSE;
-      continue;
-      }
     }
 
   /* Starting with ! specifies a negative item. It is theoretically possible
@@ -588,8 +546,7 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
     yield = FAIL;
     while (isspace((*(++ss))));
     }
-  else
-    yield = OK;
+  else yield = OK;
 
   /* If the item does not begin with '/', it might be a + item for a named
   list. Otherwise, it is just a single list entry that has to be matched.
@@ -597,7 +554,7 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
 
   if (*ss != '/')
     {
-    if (*ss == '+' && anchorptr)
+    if (*ss == '+' && anchorptr != NULL)
       {
       int bits = 0;
       int offset = 0;
@@ -605,18 +562,15 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
       unsigned int *use_cache_bits = original_cache_bits;
       uschar *cached = US"";
       namedlist_block *nb;
-      tree_node * t;
+      tree_node *t = tree_search(*anchorptr, ss+1);
 
-      if (!(t = tree_search(*anchorptr, ss+1)))
-	{
-        log_write(0, LOG_MAIN|LOG_PANIC, "unknown named%s list \"%s\"",
-          type == MCL_DOMAIN ?    " domain" :
-          type == MCL_HOST ?      " host" :
-          type == MCL_ADDRESS ?   " address" :
-          type == MCL_LOCALPART ? " local part" : "",
+      if (t == NULL)
+        log_write(0, LOG_MAIN|LOG_PANIC_DIE, "unknown named%s list \"%s\"",
+          (type == MCL_DOMAIN)?    " domain" :
+          (type == MCL_HOST)?      " host" :
+          (type == MCL_ADDRESS)?   " address" :
+          (type == MCL_LOCALPART)? " local part" : "",
           ss);
-	return DEFER;
-	}
       nb = t->data.ptr;
 
       /* If the list number is negative, it means that this list is not
@@ -628,7 +582,7 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
       because the pointer may be NULL from the start if caching is not
       required. */
 
-      if (use_cache_bits)
+      if (use_cache_bits != NULL)
         {
         offset = (nb->number)/16;
         shift = ((nb->number)%16)*2;
@@ -652,13 +606,15 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
         wasn't before. Ensure that this is passed up to the next level.
         Otherwise, remember the result of the search in the cache. */
 
-        if (!use_cache_bits)
+        if (use_cache_bits == NULL)
+          {
           *cache_ptr = NULL;
+          }
         else
           {
           use_cache_bits[offset] |= bits << shift;
 
-          if (valueptr)
+          if (valueptr != NULL)
             {
             int old_pool = store_pool;
             namedlist_cacheblock *p;
@@ -669,16 +625,16 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
             store_pool = POOL_PERM;
             p = store_get(sizeof(namedlist_cacheblock));
             p->key = string_copy(get_check_key(arg, type));
-
-
-            p->data = *valueptr ? string_copy(*valueptr) : NULL;
             store_pool = old_pool;
 
+            p->data = *valueptr;
             p->next = nb->cache_data;
             nb->cache_data = p;
-            if (*valueptr)
+            if (*valueptr != NULL)
+              {
               DEBUG(D_lists) debug_printf("data from lookup saved for "
                 "cache for %s: %s\n", ss, *valueptr);
+              }
             }
           }
         }
@@ -691,18 +647,19 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
         {
         DEBUG(D_lists) debug_printf("cached %s match for %s\n",
           ((bits & (-bits)) == bits)? "yes" : "no", ss);
-
         cached = US" - cached";
-        if (valueptr)
+        if (valueptr != NULL)
           {
-          const uschar *key = get_check_key(arg, type);
+          uschar *key = get_check_key(arg, type);
           namedlist_cacheblock *p;
-          for (p = nb->cache_data; p; p = p->next)
+          for (p = nb->cache_data; p != NULL; p = p->next)
+            {
             if (Ustrcmp(key, p->key) == 0)
               {
               *valueptr = p->data;
               break;
               }
+            }
           DEBUG(D_lists) debug_printf("cached lookup data = %s\n", *valueptr);
           }
         }
@@ -722,55 +679,35 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
 
     else
       {
-      uschar * error = NULL;
+      uschar *error;
       switch ((func)(arg, ss, valueptr, &error))
         {
         case OK:
-	  HDEBUG(D_lists) debug_printf("%s %s (matched \"%s\")\n", ot,
-	    (yield == OK)? "yes" : "no", sss);
-	  return yield;
+        HDEBUG(D_lists) debug_printf("%s %s (matched \"%s\")\n", ot,
+          (yield == OK)? "yes" : "no", sss);
+        return yield;
 
         case DEFER:
-	  if (!error)
-	    error = string_sprintf("DNS lookup of \"%s\" deferred", ss);
-	  if (ignore_defer)
-	    {
-	    HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_defer\n",
-	      error);
-	    break;
-	    }
-	  if (include_defer)
-	    {
-	    log_write(0, LOG_MAIN, "%s: accepted by +include_defer", error);
-	    return OK;
-	    }
-	  if (!search_error_message) search_error_message = error;
-	  goto DEFER_RETURN;
+        goto DEFER_RETURN;
 
-        /* The ERROR return occurs when checking hosts, when either a forward
-        or reverse lookup has failed. It can also occur in a match_ip list if a
-        non-IP address item is encountered. The error string gives details of
+        /* The ERROR return occurs only when checking hosts, when either a
+        forward or reverse lookup has failed. The error string gives details of
         which it was. */
 
         case ERROR:
-	  if (ignore_unknown)
-	    {
-	    HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_unknown\n",
-	      error);
-	    }
-	  else
-	    {
-	    HDEBUG(D_lists) debug_printf("%s %s (%s)\n", ot,
-	      include_unknown? "yes":"no", error);
-	    if (!include_unknown)
-	      {
-	      if (LOGGING(unknown_in_list))
-		log_write(0, LOG_MAIN, "list matching forced to fail: %s", error);
-	      return FAIL;
-	      }
-	    log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
-	    return OK;
-	    }
+        if (ignore_unknown)
+          {
+          HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_unknown",
+            error);
+          }
+        else
+          {
+          HDEBUG(D_lists) debug_printf("%s %s (%s)\n", ot,
+            include_unknown? "yes":"no", error);
+          if (!include_unknown) return FAIL;
+          log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
+          return OK;
+          }
         }
       }
     }
@@ -781,16 +718,16 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
   else
     {
     int file_yield = yield;       /* In case empty file */
-    uschar * filename = ss;
-    FILE * f = Ufopen(filename, "rb");
+    uschar *filename = ss;
+    FILE *f = Ufopen(filename, "rb");
     uschar filebuffer[1024];
 
     /* ot will be null in non-debugging cases, and anyway, we get better
     wording by reworking it. */
 
-    if (!f)
+    if (f == NULL)
       {
-      uschar * listname = readconf_find_option(listptr);
+      uschar *listname = readconf_find_option(listptr);
       if (listname[0] == 0)
         listname = string_sprintf("\"%s\"", *listptr);
       log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s",
@@ -838,48 +775,30 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
       switch ((func)(arg, ss, valueptr, &error))
         {
         case OK:
-	  (void)fclose(f);
-	  HDEBUG(D_lists) debug_printf("%s %s (matched \"%s\" in %s)\n", ot,
-	    yield == OK ? "yes" : "no", sss, filename);
-	  return file_yield;
+        fclose(f);
+        HDEBUG(D_lists) debug_printf("%s %s (matched \"%s\" in %s)\n", ot,
+          (yield == OK)? "yes" : "no", sss, filename);
+        return file_yield;
 
         case DEFER:
-	  if (!error)
-	    error = string_sprintf("DNS lookup of %s deferred", ss);
-	  if (ignore_defer)
-	    {
-	    HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_defer\n",
-	      error);
-	    break;
-	    }
-	  (void)fclose(f);
-	  if (include_defer)
-	    {
-	    log_write(0, LOG_MAIN, "%s: accepted by +include_defer", error);
-	    return OK;
-	    }
-	  goto DEFER_RETURN;
+        fclose(f);
+        goto DEFER_RETURN;
 
-        case ERROR:		/* host name lookup failed - this can only */
-	  if (ignore_unknown)	/* be for an incoming host (not outgoing) */
-	    {
-	    HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_unknown\n",
-	      error);
-	    }
-	  else
-	   {
-	    HDEBUG(D_lists) debug_printf("%s %s (%s)\n", ot,
-	      include_unknown? "yes":"no", error);
-	    (void)fclose(f);
-	    if (!include_unknown)
-	      {
-	      if (LOGGING(unknown_in_list))
-		log_write(0, LOG_MAIN, "list matching forced to fail: %s", error);
-	      return FAIL;
-	      }
-	    log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
-	    return OK;
-	    }
+        case ERROR:          /* host name lookup failed - this can only */
+        if (ignore_unknown)  /* be for an incoming host (not outgoing) */
+          {
+          HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_unknown",
+            error);
+          }
+        else
+         {
+          HDEBUG(D_lists) debug_printf("%s %s (%s)\n", ot,
+            include_unknown? "yes":"no", error);
+          fclose(f);
+          if (!include_unknown) return FAIL;
+          log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
+          return OK;
+          }
         }
       }
 
@@ -887,20 +806,20 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))))
     for the file, in case this is the last item in the list. */
 
     yield = file_yield;
-    (void)fclose(f);
+    fclose(f);
     }
   }    /* Loop for the next item on the top-level list */
 
 /* End of list reached: if the last item was negated yield OK, else FAIL. */
 
 HDEBUG(D_lists)
-  debug_printf("%s %s (end of list)\n", ot, yield == OK ? "no":"yes");
-return yield == OK ? FAIL : OK;
+  debug_printf("%s %s (end of list)\n", ot, (yield == OK)? "no":"yes");
+return (yield == OK)? FAIL : OK;
 
-/* Something deferred */
+/* Handle lookup defer */
 
 DEFER_RETURN:
-HDEBUG(D_lists) debug_printf("%s list match deferred for %s\n", ot, sss);
+HDEBUG(D_lists) debug_printf("%s lookup deferred for %s\n", ot, sss);
 return DEFER;
 }
 
@@ -929,8 +848,6 @@ Arguments:
                  MCL_LOCALPART when matching a local part list (address lists
                    have their own function)
                  MCL_STRING for others (e.g. list of ciphers)
-                 MCL_NOEXPAND (whose value is greater than any of them) may
-                   be added to any value to suppress expansion of the list
   caseless       TRUE for (mostly) caseless matching - passed directly to
                    match_check_string()
   valueptr       pointer to where any lookup data is to be passed back,
@@ -945,9 +862,8 @@ Returns:         OK    if matched a non-negated item
 */
 
 int
-match_isinlist(const uschar *s, const uschar **listptr, int sep,
-   tree_node **anchorptr,
-  unsigned int *cache_bits, int type, BOOL caseless, const uschar **valueptr)
+match_isinlist(uschar *s, uschar **listptr, int sep, tree_node **anchorptr,
+  unsigned int *cache_bits, int type, BOOL caseless, uschar **valueptr)
 {
 unsigned int *local_cache_bits = cache_bits;
 check_string_block cb;
@@ -956,7 +872,7 @@ cb.subject = caseless? string_copylc(s) : string_copy(s);
 cb.expand_setup = (sep > UCHAR_MAX)? 0 : -1;
 cb.use_partial = TRUE;
 cb.caseless = caseless;
-cb.at_is_special = (type == MCL_DOMAIN || type == MCL_DOMAIN + MCL_NOEXPAND);
+cb.at_is_special = (type == MCL_DOMAIN);
 if (valueptr != NULL) *valueptr = NULL;
 return  match_check_list(listptr, sep, anchorptr, &local_cache_bits,
   check_string, &cb, type, s, valueptr);
@@ -993,55 +909,48 @@ Returns:         OK     for a match
 */
 
 static int
-check_address(void *arg, const uschar *pattern, const uschar **valueptr, uschar **error)
+check_address(void *arg, uschar *pattern, uschar **valueptr, uschar **error)
 {
 check_address_block *cb = (check_address_block *)arg;
 check_string_block csb;
 int rc;
 int expand_inc = 0;
 unsigned int *null = NULL;
-const uschar *listptr;
+uschar *listptr;
 uschar *subject = cb->address;
-const uschar *s;
-uschar *pdomain, *sdomain;
+uschar *s, *pdomain, *sdomain;
 
 error = error;  /* Keep clever compilers from complaining */
 
-DEBUG(D_lists) debug_printf("address match test: subject=%s pattern=%s\n",
+DEBUG(D_lists) debug_printf("address match: subject=%s pattern=%s\n",
   subject, pattern);
 
-/* Find the subject's domain */
-
-sdomain = Ustrrchr(subject, '@');
-
-/* The only case where a subject may not have a domain is if the subject is
-empty. Otherwise, a subject with no domain is a serious configuration error. */
-
-if (sdomain == NULL && *subject != 0)
-  {
-  log_write(0, LOG_MAIN|LOG_PANIC, "no @ found in the subject of an "
-    "address list match: subject=\"%s\" pattern=\"%s\"", subject, pattern);
-  return FAIL;
-  }
-
-/* Handle a regular expression, which must match the entire incoming address.
-This may be the empty address. */
+/* Handle a regular expression, which must match the entire incoming address. */
 
 if (*pattern == '^')
   return match_check_string(subject, pattern, cb->expand_setup, TRUE,
     cb->caseless, FALSE, NULL);
 
-/* Handle a pattern that is just a lookup. Skip over possible lookup names
-(letters, digits, hyphens). Skip over a possible * or *@ at the end. Then we
-must have a semicolon for it to be a lookup. */
+/* If the subject is the empty string, the only pattern it can match (other
+than a regular expression) is the empty pattern. */
+
+if (*subject == 0) return (*pattern == 0)? OK : FAIL;
+
+/* Find the domain in the subject */
+
+sdomain = Ustrrchr(subject, '@');
+
+/* Handle the case of a pattern that is just a lookup. Skip over possible
+lookup names (letters, digits, hyphens). Skip over a possible * or *@ at
+the end. Then we must have a semicolon for it to be a lookup. */
 
 for (s = pattern; isalnum(*s) || *s == '-'; s++);
 if (*s == '*') s++;
 if (*s == '@') s++;
 
-/* If it is a straight lookup, do a lookup for the whole address. This may be
-the empty address. Partial matching doesn't make sense here, so we ignore it,
-but write a panic log entry. However, *@ matching will be honoured. */
+/* If it is a straight lookup, do a lookup for the whole address. Partial
+matching doesn't make sense here, so we ignore it, but write a panic log entry.
+However, *@ matching will be honoured. */
 
 if (*s == ';')
   {
@@ -1052,12 +961,6 @@ if (*s == ';')
     valueptr);
   }
 
-/* For the remaining cases, an empty subject matches only an empty pattern,
-because other patterns expect to have a local part and a domain to match
-against. */
-
-if (*subject == 0) return (*pattern == 0)? OK : FAIL;
-
 /* If the pattern starts with "@@" we have a split lookup, where the domain is
 looked up to obtain a list of local parts. If the subject's local part is just
 "*" (called from retry) the match always fails. */
@@ -1065,8 +968,7 @@ looked up to obtain a list of local parts. If the subject's local part is just
 if (pattern[0] == '@' && pattern[1] == '@')
   {
   int watchdog = 50;
-  const uschar *key;
-  uschar *list, *ss;
+  uschar *list, *key, *ss;
   uschar buffer[1024];
 
   if (sdomain == subject + 1 && *subject == '*') return FAIL;
@@ -1079,7 +981,7 @@ if (pattern[0] == '@' && pattern[1] == '@')
     int sep = 0;
 
     if ((rc = match_check_string(key, pattern + 2, -1, TRUE, FALSE, FALSE,
-      CUSS &list)) != OK) return rc;
+      &list)) != OK) return rc;
 
     /* Check for chaining from the last item; set up the next key if one
     is found. */
@@ -1098,7 +1000,8 @@ if (pattern[0] == '@' && pattern[1] == '@')
     /* Look up the local parts provided by the list; negation is permitted.
     If a local part has to begin with !, a regex can be used. */
 
-    while ((ss = string_nextinlist(CUSS &list, &sep, buffer, sizeof(buffer))))
+    while ((ss = string_nextinlist(&list, &sep, buffer, sizeof(buffer)))
+           != NULL)
       {
       int local_yield;
 
@@ -1273,9 +1176,9 @@ Returns:          OK    for a positive match, or end list after a negation;
 */
 
 int
-match_address_list(const uschar *address, BOOL caseless, BOOL expand,
-  const uschar **listptr, unsigned int *cache_bits, int expand_setup, int sep,
-  const uschar **valueptr)
+match_address_list(uschar *address, BOOL caseless, BOOL expand,
+  uschar **listptr, unsigned int *cache_bits, int expand_setup, int sep,
+  uschar **valueptr)
 {
 uschar *p;
 check_address_block ab;
@@ -1319,25 +1222,4 @@ return match_check_list(listptr, sep, &addresslist_anchor, &local_cache_bits,
     valueptr);
 }
 
-/* Simpler version of match_address_list; always caseless, expanding,
-no cache bits, no value-return.
-
-Arguments:
-  address         address to test
-  listptr         list to check against
-  sep             separator character for the list;
-                  may be 0 to get separator from the list;
-                  may be UCHAR_MAX+1 for one-item list
-
-Returns:          OK    for a positive match, or end list after a negation;
-                  FAIL  for a negative match, or end list after non-negation;
-                  DEFER if a lookup deferred
-*/
-
-int
-match_address_list_basic(const uschar *address, const uschar **listptr, int sep)
-{
-return match_address_list(address, TRUE, TRUE, listptr, NULL, -1, sep, NULL);
-}
-
 /* End of match.c */
